# Laravel Carmen

## Install

### Add Myriad Repository

Carmen is distributed internally with Satis. First add the repository to `composer.json`:

```json
  "repositories": [
    {
      "type": "composer",
      "url": "https://satis.myriadmobile.com"
    }
  ]
```

### Require the Dependency

Require this package with composer using the following command:

```bash
composer require myriad/carmen
```

### Install Service Provider

After updating composer, add the service provider to the `providers` array in `config/app.php`:

```php
Myriad\Carmen\CarmenServiceProvider::class,
```

### Install Facade (Optional)

If you choose, register the facade in the `aliases` array in `config/app.php`:

```php
'Carmen' => Myriad\Carmen\Facades\Carmen::class,
```

### Configuration

By default, the following environment variables are read to setup Carmen:

* `GEOTOOLS_TOKEN` => The application's unique server token
* `GEOTOOLS_SERVER` => The url to the Carmen server. Default: https://geotools.myriadapps.com/api/v1/lookup/

If environment variables are not sufficient, the configuration file can be published using:

```bash
php artisan vendor:publish --provider=myriad/carmen --tag=config
```

## Usage

Note. Examples make use of the Facade. For best practice, inject an instance of `Myriad\Carmen\Contracts\Carmen`.

### Get lat/lng from zip

To get the lat/lng from a zip as well as other attributes call:

```php
$latlng = Carmen::getLatLngFromZip($zip);
```

### Get the distance between two points

To get the distance between two points call:

```php
$distance = Carmen::getDistanceBetween($latlngArray, $otherLatlngArray, $unit);
```

$latlngArray and $otherLatlngArray must be arrays with the keys 'lat', 'lng'.  
$unit must be either 'miles' or 'kilometers'.

### Get locations within a radius of another location.

To get all locations within a radius of another location in a table in the database call:

```php
$locationsArray = Carmen::getWithin($latlngArray, $radius, $unit, $table, $latColumn, $lngColumn);
```

$latlngArray and $otherLatlngArray must be arrays with the keys 'lat', 'lng'.  
$unit must be either 'miles' or 'kilometers'.  
$table must be the name of the table.  
$latColumn must be the name of the latitude column in the table, defaults to 'lat'.  
$latColumn must be the name of the longitude column in the table, defaults to 'lng'.  
A distance attribute will be added to all objects in the result.

### Get locations outside a radius of another location.

To get all locations outside a radius of another location in a table in the database call:

```php
$locationsArray = Carmen::getOutside($latlngArray, $radius, $unit, $table, $latColumn, $lngColumn);
```

$latlngArray and $otherLatlngArray must be arrays with the keys 'lat', 'lng'.  
$unit must be either 'miles' or 'kilometers'.  
$table must be the name of the table.  
$latColumn must be the name of the latitude column in the table, defaults to 'lat'.  
$latColumn must be the name of the longitude column in the table, defaults to 'lng'.  
A distance attribute will be added to all objects in the result.  

### Get US postal codes for Fips codes.

To get all postal codes for a list of Fips codes call:

```php
$postalCodes = Carmen::getUSPostalCodesByFipsCodes($codes);
```

$codes must be an array of state and/or county Fips code strings. For example: ["38", "38059", "27005"]

### Get postal code object for postal code and optional country code.

To get the postal code object which includes attributes:
* code
* cc
* name
* state_name
* state_code
* county_name
* county_code
* community_name
* community_code
* latitude
* longitude

```php
$postalCode = Carmen::getPostalCode("58102", "US");
```
