<?php namespace Myriad\Carmen;

use GuzzleHttp\Client as Guzzle;
use GuzzleHttp\Psr7\Request;
use Illuminate\Support\Facades\DB;
use Myriad\Carmen\Contracts\Carmen;

class CarmenClient implements Carmen {

  private static $MEASUREMENTS = [
        'miles' => 3959,
        'kilometers' => 6371
  ];

  /**
   * @var \Myriad\Carmen\CarmenConfiguration
   */
  private $config;

  /**
   * @var \GuzzleHttp\Client
   */
  private $guzzle;

  /**
   * @inheritdoc
   */
  public function __construct(CarmenConfiguration $config, Guzzle $guzzle) {
    $this->config = $config;
    $this->guzzle = $guzzle;
  }

  /**
   * @inheritdoc
   */
  public function getLatLngFromZip($zip) {
    $headers = ['Authorization' => $this->getConfiguration()->getServerToken()];
    $request = new Request('GET', $this->getConfiguration()->getServerUrl() . 'postal/' . $zip, $headers);
    $response = $this->guzzle->send($request);
    $latlng = json_decode($response->getBody(), true)['data'];

    return $latlng;
  }

  /**
   * @inheritdoc
   */
  public function getDistanceBetween(array $latlng, array $otherLatlng, $unit) {
    return self::$MEASUREMENTS[$unit] * acos(cos(deg2rad($latlng['lat'])) * cos(deg2rad($otherLatlng['lat'])) * cos(deg2rad($otherLatlng['lng']) - deg2rad($latlng['lng'])) + sin(deg2rad($latlng['lat'])) * sin(deg2rad($otherLatlng['lat'])));
  }

  /**
   * @inheritdoc
   */
  public function getWithin($latlng, $radius, $unit, $table, $latColumn='lat', $lngColumn='lng') {
    return DB::select(DB::raw("SELECT *, (" . self::$MEASUREMENTS[$unit] . "* acos(cos(radians(" . $latlng['lat'] . ")) * cos(radians(" . $latColumn . ")) * cos(radians(" . $lngColumn . ") - radians(" . $latlng['lng'] . ")) + sin(radians(" . $latlng['lat'] . ")) * sin(radians(" . $latColumn . ")))) AS distance FROM " . $table . " HAVING distance < " . $radius . " ORDER BY distance"));
  }

  /**
   * @inheritdoc
   */
  public function getOutside($latlng, $radius, $unit, $table, $latColumn='lat', $lngColumn='lng') {
    return DB::select(DB::raw("SELECT *, (" . self::$MEASUREMENTS[$unit] . "* acos(cos(radians(" . $latlng['lat'] . ")) * cos(radians(" . $latColumn . ")) * cos(radians(" . $lngColumn . ") - radians(" . $latlng['lng'] . ")) + sin(radians(" . $latlng['lat'] . ")) * sin(radians(" . $latColumn . ")))) AS distance FROM " . $table . " HAVING distance > " . $radius . " ORDER BY distance"));
  }

  /**
   * @inheritdoc
   */
  public function getConfiguration() {
    return $this->config;
  }

  /**
   * @inheritdoc
   */
  public function getUSPostalCodesByFipsCodes($fipsCodes)
  {
    $fipsCodeString = str_replace(' ', '', implode(',', $fipsCodes));

    $headers = ['Authorization' => $this->getConfiguration()->getServerToken()];
    $request = new Request('GET', $this->getConfiguration()->getServerUrl() . 'fips/' . $fipsCodeString, $headers);
    $response = $this->guzzle->send($request);
    $postalCodes = json_decode($response->getBody(), true)['data'];
    return $postalCodes;
  }

  public function getPostalCode($postalCode, $countryCode = null)
  {
    $query = 'postal/' . $postalCode;

    if ($countryCode) {
      $trimmedCountryCode = strtoupper(trim($countryCode));
      $query .= "?{$trimmedCountryCode}";
    }
    $headers = ['Authorization' => $this->getConfiguration()->getServerToken()];
    $request = new Request('GET', $this->getConfiguration()->getServerUrl() . $query, $headers);
    $response = $this->guzzle->send($request);
    $postalCodeObj = json_decode($response->getBody(), true)['data'];

    return $postalCodeObj;
  }
}